# Llibreries --------------------------------------------------------------

library(timeDate)
library(timeSeries)
library(fBasics)
library(aTSA)
library(forecast)
library(tseries)
library(ggplot2)
library(lmtest)
library(fUnitRoots)
library(FitARMA)
library(strucchange)
library(reshape)
library(Rmisc)
library(tsoutliers)
library(fitdistrplus)
library(actuar)
library(car)

# Lectura de dades --------------------------------------------------------

par(mfrow=c(1,1))
serie <- ts(read.table("atur_ts.txt"), start = 1996, frequency = 12)
n <- length(serie)

# Anàlisi inicial ---------------------------------------------------------

plot.ts(serie)

# Anàlisi gràfic
plot(decompose(serie))

# Pendent
plot(aggregate(serie))

# Estacionalitat
boxplot(serie~cycle(serie))

# ACF i PACF
par(mfrow=c(2,1))
acf(serie, lag= 100)
pacf(serie, lag=100)

# Recomanacions
ndiffs(serie)
nsdiffs(serie)

# Sèrie no estacionària
# Clara estacionalitat i pendent positiu
# S'aplicaran diferencies regulars i estacionals per 
# a tindre una sèrie estacionària

# Diferències regulars ----------------------------------------------------
dserie <- diff(serie)

# Gràfics
par(mfrow=c(1,1))
plot(dserie)
plot(decompose(dserie))

# ACF i PACF
par(mfrow=c(2,1))
acf(dserie, lag=100)
pacf(dserie, lag=100)

# S'ha eliminat la tendència
par(mfrow=c(1,1))
plot(aggregate(dserie))

# La sèrie segueix sense ser estacionària
# Però ja no tenim tendència
# Presència d'un pic cap al 2008

# Diferències estacionals -------------------------------------------------

# Fem diferències estacionals per extreure el cicle
ddserie <- diff(dserie, lag = 12)
par(mfrow=c(1,1))
plot(ddserie)

# ACF i PACF
par(mfrow=c(2,1))
acf(ddserie, lag=100)
pacf(ddserie, lag =100)

# Eliminació estacionalitat
par(mfrow=c(1,1))
boxplot(ddserie~cycle(ddserie))

# Recomanacions
ndiffs(ddserie)
nsdiffs(ddserie)
auto.arima(ddserie, ic = 'aicc')

# Eliminada tendència i estacionalitat
# Sembla estacionària per el ACF i el PACF

# Presència d'outliers i pics cap al 2009 i 2020
par(mfrow=(c(1,1)))
plot(ddserie)
plot(aggregate(ddserie))
boxplot(ddserie~cycle(ddserie))

# Selecció model ----------------------------------------------------------

# S'utilitza la funció auto.arima per estimar quin podria ser el millor model
model <- auto.arima(serie, ic = "aicc", trace = TRUE,
                    d = 1, D = 1, method = "CSS-ML")
model

# SARIMA(1,1,3)(0,1,2)[12] amb par?metres
# AR1 = 0.9346
# MA1 = -0.3389
# MA2 = -0.1019
# MA3 = -0.3165
# SMA1 = -0.4140
# SMA2 = -0.1246
model <- Arima(serie,
               order = c(1,1,3),
               seasonal = list(order = c(0,1,2), period =12),
               method = "CSS-ML")

model

# Residus -----------------------------------------------------------------
resid <- model$residuals

# Residus tipificats
resid_tip <- scale(resid, center = T, scale = T)

# Gràfics
par(mfrow=c(1,1))
plot(resid)
tsdiag(model)

# ACF i PACF
par(mfrow=c(2,1))
acf(resid, lag = 100)
pacf(resid,lag = 100)

# Soroll IID: SI
Box.test(resid, type="Ljung-Box")

# Normalitat: NO
qqnorm(resid)
qqline(resid)
shapiro.test(resid)

# Histograma
par(mfrow=c(1,1))
hist(resid_tip, prob = TRUE, breaks = 50)
x <- seq(min(resid_tip), max(resid_tip), length = n)
f <- dnorm(x, mean = mean(resid_tip), sd = sd(resid_tip))
lines(x, f, col = "red", lwd = 2)

# Test Kolmogorov Smirnov amb t-Student (No s'ajusta)
pvalues = vector()
for (i in (1:30)) {
  test <- ks.test(resid_tip, "pt", df = i)
  pvalues <- append(pvalues, test$p.value)
}
pvalues
barplot(pvalues)

# Prediccions -------------------------------------------------------------

pred <- predict(model, n.ahead = 10)
pred

# Valors predits
p <- pred$pred

# Intervals 95%
p_upper <- pred$pred + 1.96 * pred$se
p_lower <- pred$pred - 1.96 * pred$se

# Gràfic
par(mfrow=c(1,1))
plot(cbind(serie, p_lower), plot.type = "single", ylab = "", type = "n")
lines(serie)
lines(p, type = "l", col = "blue")
lines(p_upper, type = "l", col = "red", lty = 2)  
lines(p_lower, type = "l", col = "red", lty = 2)  
legend("topleft", legend = c("observed data", 
                                "forecasts", "95% confidence bands"),
       lty = c(1,1,2,2),col = c("black", "blue", "red", "red"), bty = "n")

# S'ha vist que hi han potencials outliers cap al 2008 i a l'inici del 2020

# Recerca outliers --------------------------------------------------------

# Busquem els outliers de la sèrie
outliers <- tso(serie, types = c("TC", "AO", "LS", "IO"), tsmethod = "arima",
                discard.method = "en-masse",
                args.tsmethod = list(order = c(1,1,3),
                                     seasonal = list(order = c(0,1,2), 
                                                     period =12)))
model
outliers # representació gràfica
plot(outliers) # outliers trobats
outliers$fit # model

# Vector amb els tipus d'outliers
outliers_type <- outliers$outliers$type
# Outlier 1 -> Canvi temporal (TC)
# Outlier 2 -> Outlier innovacional (IO)

# Vector amb els index a la sèrie temporal dels outliers
outliers_ind <- outliers$outliers$ind
# Outlier 1 -> Observació 162
# Outlier 2 -> Observació 292

# Vector amb el temps dels outliers
outliers_time <- outliers$outliers$time
# Outlier 1 -> Juny del 2009
# Outlier 2 -> Abril del 2020

# Anàlisi outliers --------------------------------------------------------

# OUTLIER 1

outlier_o1 <- outliers(outliers_type[1], outliers_ind[1])
o1 <- outliers.effects(outlier_o1, n)

# Coeficient de l'outlier
coefhat_o1 <- outliers$outliers$coefhat[1]

# Efecte sobre la s?rie temporal de l'outlier
o1_effect <- coefhat_o1*o1

# Convertim l'efecte en una s?rie temporal
o1_effect_ts <- ts(o1_effect, frequency = frequency(serie),
                   start = start(serie))

# Plot de l'efecte
par(mfrow=c(1,1))
plot(o1_effect_ts)

# Outlier additiu amb coeficient 20674 que comença el seu efecte
#al Març de 2020. 

# OUTLIER 2

outlier_o2 <- outliers(outliers_type[2], outliers_ind[2])
o2 <- outliers.effects(outlier_o2, n, pars = coefs2poly(outliers$fit))

# Coeficient de l'outlier
coefhat_o2 <- outliers$outliers$coefhat[2]

# Efecte sobre la s?rie temporal de l'outlier
o2_effect <- coefhat_o2*o2

# Convertim l'efecte en una sèrie temporal
o2_effect_ts <- ts(o2_effect, frequency = frequency(serie),
                   start = start(serie))

# Plot de l'efecte
plot(o2_effect_ts)

# Outlier innovacional de coeficient 83443 que comença el seu efecte
# a l'Abril del 2020

# Disseny nou model -------------------------------------------------------

# S'ha estimat que és un model SARIMA(1,1,3)(0,1,2)
outliers

# Creació model

model_outliers <- outliers$fit

# Comprovació residus -----------------------------------------------------

resid <- model_outliers$residuals

# Residus tipificats
resid_tip <- scale(resid, center = T, scale = T)

# Gràfics
par(mfrow=c(1,1))
plot(resid)
tsdiag(model)

# ACF i PACF
par(mfrow=c(2,1))
acf(resid, lag = 100)
pacf(resid,lag = 100)

# Soroll IID: S?
Box.test(resid, type="Ljung-Box")

# Normalitat: NO
qqnorm(resid)
qqline(resid)
shapiro.test(resid_tip)
ks.test(resid_tip, "pnorm")

# Predicció ---------------------------------------------------------------
# Predicció TC
forecast_o1 <- outliers.effects(outlier_o1,n+12)[303:312]

# Predicció IO
forecast_o2 <- outliers.effects(outlier_o2,n+12,
                                pars = coefs2poly(model_outliers))[303:312]

regressors = cbind(forecast_o1, forecast_o2)
regressors

pred_outliers <- predict(model_outliers, n.ahead = 10, newxreg = regressors)
pred

# Valors predits
p <- pred_outliers$pred

# Intervals 95%
p_upper <- pred_outliers$pred + 1.96 * pred_outliers$se
p_lower <- pred_outliers$pred - 1.96 * pred_outliers$se

# Gràfic
par(mfrow=c(1,1))
plot(cbind(serie, p_upper), plot.type = "single", ylab = "", type = "n")
lines(serie)
lines(p, type = "l", col = "blue")
lines(p_upper, type = "l", col = "red", lty = 2)  
lines(p_lower, type = "l", col = "red", lty = 2)  
legend("topleft", legend = c("observed data", 
                             "forecasts", "95% confidence bands"),
       lty = c(1,1,2,2), col = c("black", "blue", "red", "red"), bty = "n")
pred

# Comparació prediccions --------------------------------------------------

par(mfrow=c(1,1))
plot(pred_outliers$pred, col = "blue")
lines(pred$pred, col = "red")
legend("topleft", legend = c("Prediccions sense anàlisi d'intervenció",
                             "Prediccions amb anàlisi d'intervenció"),
       lty=c(1,1), col = c("red","blue"))