# Llibreries --------------------------------------------------------------

library(timeDate)
library(timeSeries)
library(fBasics)
library(aTSA)
library(forecast)
library(tseries)
library(ggplot2)
library(lmtest)
library(fUnitRoots)
library(FitARMA)
library(strucchange)
library(reshape)
library(Rmisc)
library(tsoutliers)
library(fitdistrplus)
library(car)

# Lectura de dades --------------------------------------------------------

par(mfrow=c(1,1))
serie <- ts(read.table("establiments_hotelers_ts.txt"),
            start = 1999, frequency = 12)
n <- length(serie)

# Anàlisi inicial ---------------------------------------------------------

# Gràfic sèrie:
plot.ts(serie) 

# Anàlisi gràfic:
plot(decompose(serie))

# Tendència:
plot(aggregate(serie))

# Estacionalitat:
boxplot(serie~cycle(serie))

# ACF i PACF:
par(mfrow=c(2,1))
acf(serie, lag= 100)
pacf(serie, lag=100)

# Recomanacions de diferenciacions:
ndiffs(serie)
nsdiffs(serie)

# Sèrie no estacionària
# En primer lloc cal aplicar logarítmes ja que la variància augmenta
# al llarg del temps
# Clara estacionalitat i tend?ncia positiva
# S'aplicaran diferencies regulars i estacionals per a tindre
# una sèrie estacionària


# Logaritmes --------------------------------------------------------------
lserie <- log(serie)
lserie

# Gràfics
par(mfrow=c(1,1))
plot(lserie)
plot(decompose(lserie))

# Tendència:
plot(aggregate(lserie))

# Estacionalitat:
boxplot(serie~cycle(lserie))

# ACF i PACF
par(mfrow=c(2,1))
acf(lserie, lag=100)
pacf(lserie, lag=100)

# Recomanacions de diferenciacions:
ndiffs(lserie)
nsdiffs(lserie)

# No és estacionària
# Segueix havent tendència positiva i estacionalitat
# S'aplicaran per tant diferències regulars i estacionals

# Diferències regulars ----------------------------------------------------
dserie <- diff(lserie)

# Gràfics
par(mfrow=c(1,1))
plot(dserie)
plot(decompose(dserie))

# ACF i PACF
par(mfrow=c(2,1))
acf(dserie, lag=100)
pacf(dserie, lag=100)

# S'ha eliminat la tendència
par(mfrow=c(1,1))
plot(aggregate(dserie))

# La sèrie segueix sense ser del tot estacion?ria
# Però ja no tenim tendència

# Diferències estacionals -------------------------------------------------
ddserie <- diff(dserie, lag = 12)

# Gràfics
par(mfrow=c(1,1))
plot(ddserie)
plot(decompose(ddserie))

# ACF i PACF
par(mfrow=c(2,1))
acf(ddserie, lag=100)
pacf(ddserie, lag =100)

# S'ha eliminat la estacionalitat
par(mfrow=c(1,1))
boxplot(ddserie~cycle(ddserie))

# Eliminada tendència i estacionalitat
# Sembla estacionària per el ACF i el PACF

# Selecció model ----------------------------------------------------------

# S'utilitza la funció auto.arima per estimar quin podria ser el millor model
model <- auto.arima(lserie, ic = "aicc", trace = TRUE,
                    d = 1, D = 1, method = "CSS-ML")
model

# Desprès de provar amb altres models es decideix quedar-se
# l'ARIMA(2,1,1)(0,1,1)[12] ja que és el que oferia un AICc menor
# desprès d'aplicar l'anàlisi d'intervenció

# L'estimació dels paràmetres és la següent
# AR1 = 0.7557
# AR2 = -0.3265
# MA1 = -0.8460
# SMA1 = -0.2977
model <- arima(lserie,
               order = c(2,1,1),
               seasonal = list(order = c(0,1,1), period =12),
               method = "CSS-ML")
model

# Residus -----------------------------------------------------------------
resid <- model$residuals

# Residus tipificats
resid_tip <- scale(resid, center = T, scale = T)

# Gràfics
par(mfrow=c(1,1))
plot(resid)
tsdiag(model)

# ACF i PACF
par(mfrow=c(2,1))
acf(resid, lag = 100)
pacf(resid,lag = 100)

# Soroll IID: SI
Box.test(resid, type="Ljung-Box")

# Normalitat: NO
qqnorm(resid)
qqline(resid)
shapiro.test(resid)

# Histograma
par(mfrow=c(1,1))
hist(resid_tip, prob = TRUE, breaks = 50)
x <- seq(min(resid_tip), max(resid_tip), length = n)
f <- dnorm(x, mean = mean(resid_tip), sd = sd(resid_tip))
lines(x, f, col = "red", lwd = 2)

# Test Kolmogorov Smirnov amb t-Student (No s'ajusta)
pvalues = vector()
for (i in (1:30)) {
  test <- ks.test(resid_tip, "pt", df = i)
  pvalues <- append(pvalues, test$p.value)
}
pvalues
barplot(pvalues)

# Prediccions -------------------------------------------------------------

pred <- predict(model, n.ahead =11)

# Valors predits
p <- pred$pred

# Intervals 95%
p_upper <- pred$pred + 1.96 * pred$se
p_lower <- pred$pred - 1.96 * pred$se

# Gràfic
par(mfrow=c(1,1))
plot(cbind(lserie, p_lower), plot.type = "single", ylab = "", type = "n")
lines(lserie)
lines(p, type = "l", col = "blue")
lines(p_upper, type = "l", col = "red", lty = 2)  
lines(p_lower, type = "l", col = "red", lty = 2)  
legend("bottomleft", legend = c("observed data", 
                                "forecasts", "95% confidence bands"),
       lty = c(1,1,2,2), col = c("black", "blue", "red", "red"), bty = "n")

# S'ha vist que en les últimes observacions hi han potencials outliers així que
# es farà un anàlisi d'intervenció

# Recerca outliers --------------------------------------------------------

# Detecció outliers
outliers <- tso(lserie, types = c("TC", "AO", "LS", "IO"), tsmethod = "arima",
                discard.method = "en-masse",
                args.tsmethod = list(order = c(2, 1, 1),
                                     seasonal = list(order = c(0, 1, 1),
                                                     period = 12)))
outliers
plot(outliers) # representació gràfica
outliers$outliers # outliers trobats
outliers$fit # model

# Vector amb els tipus d'outliers
outliers_type <- outliers$outliers$type
# Outlier 1 -> Outlier additiu (AO)
# Outlier 2 -> Outlier innovacional (IO)
# Outlier 3 -> Canvi temporal (TC)
# Outlier 4 -> Canvi de nivell (LS)
# Outlier 5 -> Outlier innovacional (IO)
# Outlier 6 -> Canvi temporal (TC)
# Outlier 7 -> Outlier innovacional (IO)
# Outlier 8 -> Outlier additiu (AO)
# Outlier 9 -> Canvi de nivell (LS)
# Outlier 10 -> Outlier additiu (AO)


# Vector amb els index a la sèrie temporal dels outliers
outliers_ind <- outliers$outliers$ind
# Outlier 1 -> Observació 255
# Outlier 2 -> Observació 256
# Outlier 3 -> Observació 257
# Outlier 4 -> Observació 258
# Outlier 5 -> Observació 260
# Outlier 6 -> Observació 261
# Outlier 7 -> Observació 262
# Outlier 8 -> Observació 263
# Outlier 9 -> Observació 264
# Outlier 10 -> Observació 265

# Vector amb el temps dels outliers
outliers_time <- outliers$outliers$time
# Outlier 1 -> Març del 2020
# Outlier 2 -> Abril del 2020
# Outlier 3 -> Maig del 2020
# Outlier 4 -> Juny del 2020
# Outlier 5 -> Juliol del 2020
# Outlier 6 -> Agost del 2020
# Outlier 7 -> Setembre del 2020
# Outlier 8 -> Octubre del 2020
# Outlier 9 -> Novembre del 2020
# Outlier 10 -> Desembre del 2020


# Anàlisi outliers --------------------------------------------------------

# OUTLIER 1
outlier_1 <- outliers(outliers_type[1], outliers_ind[1])
o1 <- outliers.effects(outlier_1, n)

# Coeficient de l'outlier
coefhat_o1 <- outliers$outliers$coefhat[1]

# Efecte sobre la sèrie temporal de l'outlier
o1_effect <- coefhat_o1*o1

# Convertim l'efecte en una sèrie temporal
o1_effect_ts <- ts(o1_effect, frequency = frequency(serie),
                   start = start(serie))

# Plot de l'efecte
par(mfrow=c(1,1))
plot(o1_effect_ts)

# Outlier additiu amb coeficient -1.2374 que afecta al Març de 2020

# OUTLIER 2
outlier_2 <- outliers(outliers_type[2], outliers_ind[2])
o2 <- outliers.effects(outlier_2, n, pars = coefs2poly(outliers$fit))

# Coeficient de l'outlier
coefhat_o2 <- outliers$outliers$coefhat[2]

# Efecte sobre la sèrie temporal de l'outlier
o2_effect <- coefhat_o2*o2

# Convertim l'efecte en una sèrie temporal
o2_effect_ts <- ts(o2_effect, frequency = frequency(serie),
                   start = start(serie))

# Plot de l'efecte
plot(o2_effect_ts)

# Outlier innovacional de coeficient -7.5423 que comença el seu efecte
# a l'Abril del 2020

# OUTLIER 3

outlier_3 <- outliers(outliers_type[3], outliers_ind[3])
o3 <- outliers.effects(outlier_3, n)

# Coeficient de l'outlier
coefhat_o3 <- outliers$outliers$coefhat[3]

# Efecte sobre la sèrie temporal de l'outlier
o3_effect <- coefhat_o3*o3

# Convertim l'efecte en una sèrie temporal
o3_effect_ts <- ts(o3_effect, frequency = frequency(serie),
                   start = start(serie))

# Plot de l'efecte
plot(o3_effect_ts)

# Outlier de canvi temporal de coeficient -3.4606 que comença el seu efecte
# al Maig del 2020

# OUTLIER 4

outlier_4 <- outliers(outliers_type[4], outliers_ind[4])
o4 <- outliers.effects(outlier_4, n)

# Coeficient de l'outlier
coefhat_o4 <- outliers$outliers$coefhat[4]

# Efecte sobre la sèrie temporal de l'outlier
o4_effect <- coefhat_o4*o4

# Convertim l'efecte en una sèrie temporal
o4_effect_ts <- ts(o4_effect, frequency = frequency(serie),
                   start = start(serie))

# Plot de l'efecte
plot(o4_effect_ts)

# Outlier de canvi de nivell de coeficient 2.5783 que comença el seu efecte
# al Juny de 2020

# OUTLIER 5

outlier_5 <- outliers(outliers_type[5], outliers_ind[5])
o5 <- outliers.effects(outlier_5, n, pars = coefs2poly(outliers$fit))

# Coeficient de l'outlier
coefhat_o5 <- outliers$outliers$coefhat[5]

# Efecte sobre la sèrie temporal de l'outlier
o5_effect <- coefhat_o5*o5

# Convertim l'efecte en una sèrie temporal
o5_effect_ts <- ts(o5_effect, frequency = frequency(serie),
                   start = start(serie))

# Plot de l'efecte
plot(o5_effect_ts)

# Outlier innovacional de coeficient 0.2657 que comença el seu efecte 
# al Juliol del 2020. 

# OUTLIER 6

outlier_6 <- outliers(outliers_type[6], outliers_ind[6])
o6 <- outliers.effects(outlier_6, n)

# Coeficient de l'outlier
coefhat_o6 <- outliers$outliers$coefhat[6]

# Efecte sobre la sèrie temporal de l'outlier
o6_effect <- coefhat_o6*o6

# Convertim l'efecte en una sèrie temporal
o6_effect_ts <- ts(o6_effect, frequency = frequency(serie),
                   start = start(serie))

# Plot de l'efecte
plot(o6_effect_ts)

# Outlier de canvi temporal de coeficient 0.0739 que comença el seu efecte 
# a l'Agost del 2020.

# OUTLIER 7

outlier_7 <- outliers(outliers_type[7], outliers_ind[7])
o7 <- outliers.effects(outlier_7, n, pars = coefs2poly(outliers$fit))

# Coeficient de l'outlier
coefhat_o7 <- outliers$outliers$coefhat[7]

# Efecte sobre la sèrie temporal de l'outlier
o7_effect <- coefhat_o7*o7

# Convertim l'efecte en una sèrie temporal
o7_effect_ts <- ts(o7_effect, frequency = frequency(serie),
                   start = start(serie))

# Plot de l'efecte
plot(o7_effect_ts)

# Outlier innovacional de coeficient -0.7647 que comença el seu efecte 
# al Setembre del 2020. 

# OUTLIER 8

outlier_8 <- outliers(outliers_type[8], outliers_ind[8])
o8 <- outliers.effects(outlier_8, n)

# Coeficient de l'outlier
coefhat_o8 <- outliers$outliers$coefhat[8]

# Efecte sobre la sèrie temporal de l'outlier
o8_effect <- coefhat_o8*o8

# Convertim l'efecte en una sèrie temporal
o8_effect_ts <- ts(o8_effect, frequency = frequency(serie),
                   start = start(serie))

# Plot de l'efecte
plot(o8_effect_ts)

# Outlier additiu de coeficient -1.2940 que afecta al Octubre del 2020.

# OUTLIER 9

outlier_9 <- outliers(outliers_type[9], outliers_ind[9])
o9 <- outliers.effects(outlier_9, n)

# Coeficient de l'outlier
coefhat_o9 <- outliers$outliers$coefhat[9]

# Efecte sobre la sèrie temporal de l'outlier
o9_effect <- coefhat_o9*o9

# Convertim l'efecte en una sèrie temporal
o9_effect_ts <- ts(o9_effect, frequency = frequency(serie),
                   start = start(serie))

# Plot de l'efecte
plot(o9_effect_ts)

# Outlier canvi de nivell de coeficient -1.649 que comença el seu efecte 
# al Novembre del 2020.

# OUTLIER 10

outlier_10 <- outliers(outliers_type[10], outliers_ind[10])
o10 <- outliers.effects(outlier_10, n)

# Coeficient de l'outlier
coefhat_o10 <- outliers$outliers$coefhat[10]

# Efecte sobre la sèrie temporal de l'outlier
o10_effect <- coefhat_o10*o10

# Convertim l'efecte en una sèrie temporal
o10_effect_ts <- ts(o10_effect, frequency = frequency(serie),
                    start = start(serie))

# Plot de l'efecte
plot(o10_effect_ts)

# Outlier additiu de coeficient 0.2878 que afecta al Desembre del 2020.

# Disseny nou model -------------------------------------------------------

# S'ha estimat que és un model SARIMA(2,1,1)(0,1,1)
# AR1 = -0.0949
# AR2 = 0.0654
# MA1 = -0.6661
# SMA1 = -0.9997
model_outliers <- outliers$fit
model_outliers

# Comprovació residus -----------------------------------------------------

resid <- model_outliers$residuals

# Residus tipificats
resid_tip <- scale(resid, center = T, scale = T)

# Gràfics
par(mfrow=c(1,1))
plot(resid)
tsdiag(model)

# ACF i PACF
par(mfrow=c(2,1))
acf(resid, lag = 100)
pacf(resid,lag = 100)

# Soroll IID: S?
Box.test(resid, type="Ljung-Box")

# Normalitat: NO
qqnorm(resid)
qqline(resid)
shapiro.test(resid_tip)
ks.test(resid_tip, "pnorm")

# Predicció ---------------------------------------------------------------

# Efecte dels outliers en les prediccions (11 futures observacions)

# Outlier 1: 
forecast_o1 <- outliers.effects(outlier_1,n+11)[266:276]
forecast_o1

# Outlier 2:
forecast_o2 <- outliers.effects(outlier_2,n+11,
                                pars = coefs2poly(model_outliers))[266:276]
forecast_o2

# Outlier 3:
forecast_o3 <- outliers.effects(outlier_3,n+11)[266:276]
forecast_o3

# Outlier 4:
forecast_o4 <- outliers.effects(outlier_4,n+11)[266:276]
forecast_o4

# Outlier 5:
forecast_o5 <- outliers.effects(outlier_5,n+11,
                                pars = coefs2poly(model_outliers))[266:276]
forecast_o5

# Outlier 6:
forecast_o6 <- outliers.effects(outlier_6,n+11)[266:276]
forecast_o6

# Outlier 7:
forecast_o7 <- outliers.effects(outlier_7,n+11,
                                pars = coefs2poly(model_outliers))[266:276]
forecast_o7

# Outlier 8:
forecast_o8 <- outliers.effects(outlier_8,n+11)[266:276]
forecast_o8

# Outlier 9:
forecast_o9 <- outliers.effects(outlier_9,n+11)[266:276]
forecast_o9

# Outlier 10:
forecast_o10 <- outliers.effects(outlier_10,n+11)[266:276]
forecast_o10

# Efecte futur dels outliers
regressors <- cbind(forecast_o1, forecast_o2, forecast_o3, forecast_o4,
                    forecast_o5, forecast_o6, forecast_o7, forecast_o8,
                    forecast_o9, forecast_o10)

# Utilitzem predict
pred <- predict(outliers$fit, n.ahead = 11, newxreg = regressors)

# Valors predits
p <- pred$pred

# Intervals 95%
p_upper <- pred$pred + 1.96 * pred$se
p_lower <- pred$pred - 1.96 * pred$se

# Gràfic
par(mfrow=c(1,1))
plot(cbind(lserie, p_lower), plot.type = "single", ylab = "", type = "n")
lines(lserie)
lines(p, type = "l", col = "blue")
lines(p_upper, type = "l", col = "red", lty = 2)  
lines(p_lower, type = "l", col = "red", lty = 2)  
legend("bottomleft", legend = c("observed data", 
                             "forecasts", "95% confidence bands"),
       lty = c(1,1,2,2), col = c("black", "blue", "red", "red"), bty = "n")

# Prediccions finals (exp)
p <- exp(p)
p_upper <- exp(p_upper)
p_lower <- exp(p_lower)

# Gràfic definitiu
par(mfrow=c(1,1))
plot(cbind(serie, p_lower), plot.type = "single", ylab = "", type = "n")
lines(serie)
lines(p, type = "l", col = "blue")
lines(p_upper, type = "l", col = "red", lty = 2)  
lines(p_lower, type = "l", col = "red", lty = 2)  
legend("topleft", legend = c("observed data", 
                                "forecasts", "95% confidence bands"),
       lty = c(1,1,2,2), col = c("black", "blue", "red", "red"), bty = "n")

# Ampliació ---------------------------------------------------------------
forecast_effects_o1 <- coefhat_o1 * forecast_o1
forecast_effects_o2 <- coefhat_o2 * forecast_o2
forecast_effects_o3 <- coefhat_o3 * forecast_o3
forecast_effects_o4 <- coefhat_o4 * forecast_o4
forecast_effects_o5 <- coefhat_o5 * forecast_o5
forecast_effects_o6 <- coefhat_o6 * forecast_o6
forecast_effects_o7 <- coefhat_o7 * forecast_o7
forecast_effects_o8 <- coefhat_o8 * forecast_o8
forecast_effects_o9 <- coefhat_o9 * forecast_o9
forecast_effects_o10 <- coefhat_o10 * forecast_o10

forecast_effects <- cbind(forecast_effects_o1,
                          forecast_effects_o2,
                          forecast_effects_o3,
                          forecast_effects_o4,
                          forecast_effects_o5,
                          forecast_effects_o6,
                          forecast_effects_o7,
                          forecast_effects_o8,
                          forecast_effects_o9,
                          forecast_effects_o10)


forecast_effects
total_effects <- rowSums(forecast_effects) 
total_effects <- ts(total_effects, frequency = 12, start = c(2021,2))
plot(total_effects)